#include "StdAfx.h"
#include "PluginExample.h"

#include <gdiplus.h>
using namespace Gdiplus;
#pragma comment (lib,"Gdiplus.lib")

#include <Windows.h>
#include <MMSystem.h>
#pragma comment(lib, "Winmm.lib")
#define _USE_MATH_DEFINES
#include <math.h>

TRenderExampleType::TRenderExampleType(void)
{

}

//-----------------------------------------------------------------------------

TRenderExampleType::~TRenderExampleType(void)
{

}

//-----------------------------------------------------------------------------

IPKeyRender* TRenderExampleType::CreateInstance(unsigned int Type, WPARAM Param, unsigned int OLED)
{
	IPKeyRender* render = new TRenderExample(this,Type,Param,OLED);
	Plugins.insert(render);
	return render;
}

//-----------------------------------------------------------------------------

void TRenderExampleType::FreeInstance(IPlugin* plg)
{
	if (!plg)
		return;
	Plugins.erase((IPKeyRender*)plg);
	delete plg;
}

//-----------------------------------------------------------------------------

void TRenderExampleType::ShowPluginWindow(IPKeyRender* plg)
{

}

//-----------------------------------------------------------------------------

HWND TRenderExampleType::GetGUI()
{
	return NULL;
}

//-----------------------------------------------------------------------------

void TRenderExampleType::RedrawControl()
{

}

//-----------------------------------------------------------------------------

void TRenderExampleType::Init(IAppControl* fb)
{
	IPluginType::Init(fb);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

TRenderExample::TRenderExample(IPluginType* pt, unsigned int Type, WPARAM Param, unsigned int OLED)
	:IPKeyRender(pt,Type,Param,OLED)
{

}

//-----------------------------------------------------------------------------

TRenderExample::~TRenderExample(void)
{
}

//-----------------------------------------------------------------------------

HWND TRenderExample::GetPluginWindow()
{
	IPKeyRenderType* type = GetOwner();
	TRenderExampleType* Render = dynamic_cast<TRenderExampleType*>(type);
	if (Render)
	{
		return Render->GetGUI();
	}
	return NULL;
}

void TRenderExample::Draw(HDC dest, size_t key, unsigned int width, unsigned int height)
{
	Graphics graph(dest);	
	unsigned int time = timeGetTime();
	double r = (sin((double)time / 1000.0)+1.0)*127.0;

	SolidBrush br(Color((BYTE)r,128,128));
	graph.FillRectangle(&br,0,0,width,height);
}

//-----------------------------------------------------------------------------

void TRenderExample::RedrawGUI()
{
	IPKeyRenderType* owner = GetOwner();
	TRenderExampleType* crender = (TRenderExampleType*)owner;
	if (crender)
	{
		crender->RedrawControl();
	}
}

//-----------------------------------------------------------------------------

void TRenderExample::Serialize(ISerializer* srlzr)
{

}

//-----------------------------------------------------------------------------

void TRenderExample::Deserialize(const char* Data)
{

}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

TConditionExampleType::TConditionExampleType()
{
	// Prepare window class
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_DBLCLKS;
	wcex.lpfnWndProc	= DefWindowProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= GetModuleHandle(NULL);
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= L"ExampleControl";
	wcex.hIconSm		= NULL;

	RegisterClassEx(&wcex);
}

//-----------------------------------------------------------------------------

TConditionExampleType::~TConditionExampleType()
{

}

//-----------------------------------------------------------------------------

IPCondition*	TConditionExampleType::CreateInstance(unsigned int Type, WPARAM Param, unsigned int OLED)
{
	IPCondition* render = new TConditionExample(this,Type,Param,OLED);
	Plugins.insert(render);
	return render;
}

void TConditionExampleType::FreeInstance(IPlugin* plg)
{
	if (!plg)
		return;
	Plugins.erase((IPCondition*)plg);
	delete plg;
}

//-----------------------------------------------------------------------------

TConditionExample::TConditionExample(IPluginType* pt, unsigned int Type, WPARAM Param, unsigned int OLED) : 
	IPCondition(pt,Type,Param,OLED), 
	Active(false),
	Window(nullptr),
	Button(nullptr)
{
	// Subscribe on 'A' key press notification
	AppControl->SubscribeOnKeyboard('A',this);
	// Lets create a main window
	Window = CreateWindowW(L"ExampleControl",L"Sample", WS_CLIPCHILDREN, 0, 0, 200,20, nullptr, 0,GetModuleHandle(NULL),NULL);
	// Now place a button on it
	Button = CreateWindow(L"BUTTON", L"Press me", WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON | WS_TABSTOP,
		10, 2, 100, 22, Window, 0, GetModuleHandle(NULL), NULL);
	// Now set the current system font on it
	NONCLIENTMETRICS ncm;
	ncm.cbSize = sizeof(NONCLIENTMETRICS);
	::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm, 0);			// Get system font metrics
	HFONT hFont = ::CreateFontIndirect(&ncm.lfMessageFont);										// Create the font for a button, using this metrics

	::SendMessage(Button, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));						// Set the font to button
}

//-----------------------------------------------------------------------------

TConditionExample::~TConditionExample()
{
	// cleanup
	AppControl->UnsubscribeKey('A',this);
	if (Button)
		DestroyWindow(Button);
	if (Window)
		DestroyWindow(Window);
}

//-----------------------------------------------------------------------------

bool TConditionExample::IsActive()
{
	// This shows if the condition is active or not
	return Active;
}

//-----------------------------------------------------------------------------

void TConditionExample::HookTrigger(DWORD message, WPARAM wParam, LPARAM lParam)
{
	// Message about 'A' letter was recieved
	switch (message)
	{
	case KM_KEYDOWN:
		Active = true;
		break;
	case KM_KEYUP:
		Active = false;
		break;
	}
	
}

//-----------------------------------------------------------------------------

void TConditionExample::SetBackgroundColor(COLORREF color)
{
	if (Window)
	{
		HBRUSH brush = CreateSolidBrush(color);
		// set window class background - for this example. The real plugin should process WM_ERASEBKGND instead
		SetClassLongPtr(Window, GCLP_HBRBACKGROUND, (LONG)brush);
	}	
}

//-----------------------------------------------------------------------------

void TConditionExample::RedrawGUI()
{
	::InvalidateRect(Window,NULL,TRUE);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

TActionExampleType::TActionExampleType()
{

}

//-----------------------------------------------------------------------------

TActionExampleType::~TActionExampleType()
{

}

//-----------------------------------------------------------------------------

void TActionExampleType::ShowPluginWindow(IPAction* plg)
{
	// nothing here, unless you have a window with options
}

//-----------------------------------------------------------------------------

IPAction* TActionExampleType::CreateInstance(unsigned int Type, WPARAM Param, unsigned int OLED)
{
	IPAction* render = new TActionExample(this,Type,Param,OLED);
	Plugins.insert(render);
	return render;
}

//-----------------------------------------------------------------------------

void TActionExampleType::FreeInstance(IPlugin* plg)
{
	if (!plg)
		return;
	Plugins.erase((IPAction*)plg);
	delete plg;
}

//-----------------------------------------------------------------------------

TActionExample::TActionExample(IPluginType* pt, unsigned int Type, WPARAM Param, unsigned int OLED) : 
	IPAction(pt,Type,Param,OLED)
{

}

//-----------------------------------------------------------------------------

TActionExample::~TActionExample()
{

}

//-----------------------------------------------------------------------------

void TActionExample::Action(DWORD message, WPARAM wParam, LPARAM lParam)
{
	// The action runs here
	// Let's show a messagebox to test it
	MessageBox(NULL,L"That was an action example!",L"Popularis action window",MB_OK);
}

//-----------------------------------------------------------------------------
