/*
	Art. Lebedev Studio (www.artlebedev.com)
	date:		2012
	author:		Grigoriy Odegov

	This file provides declarations of plugin interfaces for Popularis Configurator software
*/
#pragma once
#ifndef IAPPCONTROLH
#define IAPPCONTROLH

#include <Windows.h>
#include <string>
#include <vector>

enum CoreMessages
{
	CM_INIT		= WM_USER+100+ 0x01,
	CM_UPDATE	= WM_USER+100+ 0x02,
	CM_UPDATE_KEYS = WM_USER + 100 + 0x03
};
class IPlugin;
class IPCondition;
class IOptimusDevice;
class TPluginBounding;

// The application controller class interface
class IAppControl
{
public:
	virtual void			UpdateRequest(IPlugin* sender)=0;			// Requests update for every display, which has a "sender" plugin attached. If sender is NULL, all displays are updated 

	virtual unsigned int	VirtualKeyToCode(unsigned int KeyCode)=0;	// converts Windows virtual key code to popularis key code (the key index in the internal keylist)

	virtual unsigned int	OledToCode(unsigned int oled)=0;			// converts display index to popularis key code
	virtual unsigned int	KeyIndexToCode(unsigned int key)=0;			// converts

	virtual unsigned int	CodeToVK(unsigned int code)=0;				// converts popularis key code to Windows virtual key code
	virtual unsigned int	CodeToOled(unsigned int code)=0;			// converts popularis key code to OLED display index
	virtual unsigned int	CodeToIndex(unsigned int code)=0;			// 

	virtual unsigned int	OLEDToVK(unsigned int code)=0;				// converts OLED display index to appropriate virtual key code, assigned to the display's button
	virtual unsigned char	OLEDToScanCode(unsigned int OLED)=0;
	virtual void			OLEDToScanCodeSequence(unsigned int OLED, unsigned char* ScanArr, unsigned int Size)=0;

	virtual void			SubscribeOnKeyboard(unsigned int Key, IPCondition* condition)=0;	// Subscribe on keyboard event (keyboard key press etc.) Key - Windows virtual key code, which will trigger the event, condition - condition plugin instance, which will recieve an event
	virtual void			SubscribeWindow(HWND Win, IPCondition* Condition)=0;				// Subscribe on window event (active/focus change, etc.) Win - handle to the window, which triggers the event.
	virtual void			SubscribeLocale(HKL Locale, IPCondition* Condition)=0;				// Subscribe on language change event. Local - the Windows locale descriptor.
	virtual void			SubscribeOnProcess(const wchar_t *Proc, IPCondition* Condition)=0;	// Subscribe on process named Proc start.

	virtual void			UnsubscribeKey(unsigned int Key, IPCondition* condition)=0;			// Unsubscribe plugin from keyboard events, triggered by Key
	virtual void			UnsubscribeWindow(HWND Win, IPCondition* Condition)=0;
	virtual void			UnsubscribeLocale(HKL Locale, IPCondition* Condition)=0;
	virtual void			UnsubscribeOnProcess(const wchar_t *Proc, IPCondition* Condition)=0;

	virtual void			SetScanCodes(int disp, std::vector<unsigned char> &data)=0;
	virtual void			WriteScanCodes()=0;

	virtual void			PostCoreMessage(UINT message, WPARAM wParam, LPARAM lParam)=0;		// Send an async message to configurator message pump
	virtual IOptimusDevice* GetCurrentDevice()=0;												// Get a current device interface
	virtual void			LockDevice()=0;														
	virtual void			UnlockDevice()=0;
	virtual void			PluginStateUpdate(IPlugin* Self)=0;

	virtual bool			IsKeyboardDevicePresent(HANDLE handle)=0;
	virtual unsigned int	GetPopularisKeyboardCount()=0;
	virtual void			AddWindowToQueue(HWND wnd)=0;
	virtual void			RemoveWindowFromQueue(HWND wnd)=0;
};

#endif